/**@file

This library provides functions for opening the fwWebBrowser: a web browser based
on the WinCCOA's (and Qt's) built-in webview widget. This browser has basic
navigation functionality and features tabbed view.
*/


/**
    Opens a new web browser as a child panel, with a specified URL

    @param sLink specifies the URL to open
    @param sLabel (optional) allows to define the name of the tab in which the URL is open
    @param sWindowName (optional) allows to define the window name (it will appear in the task bar)
    @param bNavigationButton (optional) if true, navigation buttons will be shown
    @param bUrlBarVisibility (optional) if true, URL bar will be displayed
    @param bAddressEditable (optional) if true, the address in URL bar may be edited
    @param bNewTabButton (optional) if true, a button to open a new tab will be shown
    @param sDefaultPage (optional) declares the default (home) page
*/
void fwWebBrowser_showStandaloneWindow(string sLink, dyn_string &exceptionInfo,
                                  string sLabel = "",
                                  string sWindowName = "",
                                  bool bNavigationButton = 1,
                                  bool bUrlBarVisibility = 1,
                                  bool bAdressEditable = 1,
                                  bool bNewTabButton = 1,
                                  string sDefaultPage = "http://home.cern",
                                  bool bOpenInNewModule =1)
{
	bool hasFwViewer=isFunctionDefined("fwViewer_showStandaloneWindow");

	if (!hasFwViewer) {
	    // try if we could load it dynamically
	    string componentVersion;
	    if(fwInstallation_isComponentInstalled("fwViewer", componentVersion)) {
		bool retVal;
		int rc=evalScript(retVal,"#uses \"fwViewer/fwViewer_API.ctl\" "+
		                         "int main(){return isFunctionDefined(\"fwViewer_showStandaloneWindow\");}", makeDynString());
		if (rc==0 && retVal==true) {
		    hasFwViewer=true;
		}
	    }
	}

	string componentVersion;

	if(hasFwViewer) {
		fwViewer_showStandaloneWindow(sLink, exceptionInfo, sLabel, sWindowName, bNavigationButton, bUrlBarVisibility, bAdressEditable, bNewTabButton, sDefaultPage, bOpenInNewModule);
	} else {
		fwGeneral_openInExternalBrowser(sLink, exceptionInfo);
	}
}



